#include <stdio.h>
#include <avr\io.h>
#define F_CPU 32000000UL
#include <util\delay.h>
#include <inttypes.h>

/********************** initialization **************************************************/
void Config32MHzClock(void)
{
  CCP = CCP_IOREG_gc; //Security Signature to modify clock 
  // initialize clock source to be 32MHz internal oscillator (no PLL)
  OSC.CTRL = OSC_RC32MEN_bm; // enable internal 32MHz oscillator
  while(!(OSC.STATUS & OSC_RC32MRDY_bm)); // wait for oscillator ready
  CCP = CCP_IOREG_gc; //Security Signature to modify clock 
  CLK.CTRL = 0x01; //select sysclock 32MHz osc
}

void init(void)
{
  Config32MHzClock();

  CLK.PSCTRL = 0x00; 			// no division on peripheral clock

  PORTC.DIR |= (1<<3) | (1<<0); // set PORTC:3 transmit pin as output
  PORTC.OUT |= (1<<3);          // set PORTC:3 hi 
  USARTC0.BAUDCTRLA = 207; 		// 9600b  (BSCALE=207,BSEL=0)

  USARTC0.CTRLB = USART_TXEN_bm | USART_RXEN_bm; // enable tx and rx on USART

  PORTE.DIR = 0xFF; // led port
  PORTE.OUT = 0xFF; // leds off
}

/********************** serial communication ********************************************/
void chPutChar(uint8_t zeichen)
{
    USARTC0.DATA = zeichen; 						// transmit 
	if(!(USARTC0.STATUS&USART_DREIF_bm))
		while(!(USARTC0.STATUS & USART_TXCIF_bm));	// wait for TX complete
  	USARTC0.STATUS |= USART_TXCIF_bm;  				// clear TX interrupt flag
}
uint8_t chGetChar()
{

	while(!(USARTC0.STATUS&USART_RXCIF_bm));  		// wait for RX complete
	PORTE.OUTTGL = (1<<7); 							// toogle indicator led

  	return USARTC0.DATA;
}
uint8_t chCharReady()
{
	return (USARTC0.STATUS&USART_RXCIF_bm);
}

void printString(char *data)
{
	unsigned char i = 0;
	while(data[i]!=0x00) chPutChar(data[i++]);
}

void PrintInt(int16_t wert)
{  	
	char text[6];
	itoa(wert,text,10);
	printString(text);
}
void PrintTextInt(char *str,int16_t wert)
{  	
	char text[6];
	printString(str);
	PrintInt(wert);
	printString("\n\r");
}





/********************** main programm ***************************************************/

#define BUFFERLENGTH 256

uint8_t Buffer[BUFFERLENGTH];

int main(void)
{
	uint16_t n,adr=0;
	uint8_t command;


	init();
	
	PORTE.OUTTGL = (1<<7); 							// toogle indicator led

	printString("XPLAIN LOGIKANALYZER \n\r");
	_delay_ms(1000);
	  	
	PORTD.DIR = 0x00; // analyzer input port

	while(1)
	{
		
		while(!chCharReady())
		{
			PORTE.OUT=~PORTD.IN; // status des XPLAIN Eingangsports an LEDs anzeigen
		}
		command=chGetChar();
		switch(command)
		{
			// Logikwerte vom XPLAIN Port D mglichst schnell einlesen
			case 'd': 
			{
				adr=0;
				for(n=0;n<BUFFERLENGTH;n++)
				{
					Buffer[adr++]=PORTD.IN; // bits lesen
				}
				adr=0;

			}break;
			// wert an PC senden
			case 'a':
			{
				chPutChar(Buffer[adr++]);
				if(adr>BUFFERLENGTH) adr=0;
			}break;
			// wert an PC senden
			case 's':
			{
				chPutChar(Buffer[adr++]);
				if(adr>BUFFERLENGTH) adr=0;
			}break;
			default:;
		}

	}
}

